package com.landray.kmss.sys.attachment.util;

import com.landray.kmss.util.StringUtil;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.net.URLConnection;
import java.net.URL;
import java.awt.image.BufferedImage;
import org.apache.commons.io.IOUtils;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.awt.Image;
import java.io.ByteArrayOutputStream;
import javax.imageio.ImageIO;
import java.io.IOException;
import com.landray.kmss.sys.attachment.model.SysAttBase;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

public class AttImageUtils
{
    private static final Logger log;
    private static final ThreadLocal<Long> startTimes;
    
    static {
        log = LoggerFactory.getLogger(AttImageUtils.class);
        startTimes = new ThreadLocal<Long>();
    }
    
    public static void resetInputStream(final SysAttBase sysAttMain) throws IOException {
        if ("pic".equals(sysAttMain.getFdAttType())) {
            if (sysAttMain.getWidth() != null && sysAttMain.getHeight() != null && sysAttMain.getWidth() > 0 && sysAttMain.getHeight() > 0) {
                if (AttImageUtils.log.isDebugEnabled()) {
                    AttImageUtils.log.debug("开始执行图片压缩 model:" + sysAttMain.getFdModelName() + " key:" + sysAttMain.getFdKey());
                    AttImageUtils.startTimes.set(System.currentTimeMillis());
                }
                comprass(sysAttMain);
                if (AttImageUtils.log.isDebugEnabled()) {
                    AttImageUtils.log.debug("图片压缩结束 共用时间：'" + (System.currentTimeMillis() - AttImageUtils.startTimes.get()) + "'毫秒...");
                }
            }
            else {
                AttImageUtils.log.debug("图片没设定大小，不进行压缩！");
            }
        }
        else {
            AttImageUtils.log.debug("不是图片，不进行处理！");
        }
    }
    
    private static void comprass(final SysAttBase sysAttMain) throws IOException {
        final InputStream in = sysAttMain.getInputStream();
        final int width = sysAttMain.getWidth();
        final int height = sysAttMain.getHeight();
        boolean proportion = true;
        if ("false".equals(sysAttMain.getProportion())) {
            proportion = false;
        }
        final BufferedImage img = ImageIO.read(in);
        final int imgW = img.getWidth();
        final int imgH = img.getHeight();
        if (AttImageUtils.log.isDebugEnabled()) {
            AttImageUtils.log.debug("要求大小：（" + width + "x" + height + "）实际大小：（" + imgW + "x" + imgH + "）");
        }
        final ByteArrayOutputStream out = new ByteArrayOutputStream();
        if (width >= imgW && height >= imgH) {
            if (AttImageUtils.log.isDebugEnabled()) {
                AttImageUtils.log.debug("图片小于规定大小，不进行原始图片压缩！");
            }
            if (!ImageCompressUtils.getInstance().isBMP(sysAttMain.getFdFileName())) {
                img.flush();
                if (in.markSupported()) {
                    in.reset();
                }
                return;
            }
            if (AttImageUtils.log.isDebugEnabled()) {
                AttImageUtils.log.debug("图片是‘BMP’格式，转换成‘JPEG’格式。否则Flash无法显示。");
            }
            ImageCompressUtils.getInstance().compressImage(img, out, imgW, imgH, "jpeg");
        }
        else {
            if (AttImageUtils.log.isDebugEnabled()) {
                AttImageUtils.log.debug("开始压缩图片：" + sysAttMain.getFdFileName());
            }
            ImageCompressUtils.getInstance().compressImage(img, out, width, height, proportion);
        }
        final ByteArrayInputStream newIn = new ByteArrayInputStream(out.toByteArray());
        sysAttMain.setInputStream((InputStream)newIn);
        sysAttMain.setFdSize(Double.valueOf((double)newIn.available()));
        img.flush();
        IOUtils.closeQuietly(in);
    }
    
    public static boolean checkUrlResourceExist(final String netUrl) throws IOException {
        InputStream in = null;
        try {
            final URL url = new URL(netUrl);
            final URLConnection uc = url.openConnection();
            in = uc.getInputStream();
        }
        catch (final Exception ex) {
            return false;
        }
        finally {
            if (in != null) {
                in.close();
            }
        }
        if (in != null) {
            in.close();
        }
        return true;
    }
    
    public static String getCurDnsHostContextPath(final HttpServletRequest request) {
        String host = "";
        if (request != null) {
            final String scheme = request.getScheme();
            final String serverName = request.getServerName();
            host = String.valueOf(scheme) + "://" + serverName;
            final int serverPort = request.getServerPort();
            if (serverPort != 80) {
                host = String.valueOf(host) + ":" + serverPort;
            }
            final String contextPath = request.getContextPath();
            host = String.valueOf(host) + contextPath;
        }
        return host;
    }
    
    public static boolean cacheConsulation(final HttpServletRequest request, final HttpServletResponse response, final String contentType, final String fileName) {
        boolean rtnVal = false;
        final String filethumb = request.getParameter("filethumb");
        Boolean isThumb = "yes".equals(filethumb);
        if (!isThumb) {
            final String filekey = request.getParameter("filekey");
            if (StringUtil.isNotNull(filekey)) {
                isThumb = (filekey.indexOf("image2thumbnail") >= 0);
            }
        }
        if (contentType.indexOf("image") > -1 || (fileName.endsWith(".png") && "application/octet-stream".equals(contentType)) || isThumb) {
            if (StringUtil.isNotNull(request.getHeader("If-Modified-Since"))) {
                response.reset();
                response.setStatus(304);
                rtnVal = true;
            }
            else {
                final long expires = 604800L;
                final long nowTime = System.currentTimeMillis();
                response.addDateHeader("Last-Modified", nowTime + expires);
                response.addDateHeader("Expires", nowTime + expires * 1000L);
                response.setContentType(contentType);
                response.setHeader("Cache-Control", "max-age=" + expires);
            }
        }
        return rtnVal;
    }
}
